@@ -15,7 +15,7 @@ async def check_fork_next_block(
         potential_peek = uint32(our_peak_height + 1)
         # This is the fork point in SES in the case where no fork was detected
         if blockchain.get_peak_height() is not None and fork_point_height == max_fork_ses_height:
-            for peer in peers_with_peak:
+            for peer in peers_with_peak.copy():
                 if peer.closed:
                     peers_with_peak.remove(peer)
                     continue
