@@ -3,6 +3,7 @@ from __future__ import annotations
 import dataclasses
 import logging
 import time
+import traceback
 from secrets import token_bytes
 from typing import Any, Dict, List, Optional, Set, Tuple
 
@@ -11,14 +12,14 @@ from blspy import AugSchemeMPL, G2Element
 from chinilla.consensus.cost_calculator import NPCResult
 from chinilla.full_node.bundle_tools import simple_solution_generator
 from chinilla.full_node.mempool_check_conditions import get_name_puzzle_conditions
+from chinilla.types.announcement import Announcement
 from chinilla.types.blockchain_format.coin import Coin
 from chinilla.types.blockchain_format.program import Program
 from chinilla.types.blockchain_format.sized_bytes import bytes32
-from chinilla.types.announcement import Announcement
 from chinilla.types.coin_spend import CoinSpend
+from chinilla.types.condition_opcodes import ConditionOpcode
 from chinilla.types.generator_types import BlockGenerator
 from chinilla.types.spend_bundle import SpendBundle
-from chinilla.types.condition_opcodes import ConditionOpcode
 from chinilla.util.byte_types import hexstr_to_bytes
 from chinilla.util.condition_tools import conditions_dict_for_solution, pkm_pairs_for_conditions_dict
 from chinilla.util.hash import std_hash
@@ -29,12 +30,15 @@ from chinilla.wallet.cat_wallet.cat_utils import (
     CAT_MOD,
     SpendableCAT,
     construct_cat_puzzle,
-    unsigned_spend_bundle_for_spendable_cats,
     match_cat_puzzle,
+    unsigned_spend_bundle_for_spendable_cats,
 )
-from chinilla.wallet.derivation_record import DerivationRecord
 from chinilla.wallet.cat_wallet.lineage_store import CATLineageStore
+from chinilla.wallet.coin_selection import select_coins
+from chinilla.wallet.derivation_record import DerivationRecord
 from chinilla.wallet.lineage_proof import LineageProof
+from chinilla.wallet.outer_puzzles import AssetType
+from chinilla.wallet.puzzle_drivers import PuzzleInfo
 from chinilla.wallet.payment import Payment
 from chinilla.wallet.puzzles.tails import ALL_LIMITATIONS_PROGRAMS
 from chinilla.wallet.puzzles.p2_delegated_puzzle_or_hidden_puzzle import (
@@ -42,14 +46,12 @@ from chinilla.wallet.puzzles.p2_delegated_puzzle_or_hidden_puzzle import (
     calculate_synthetic_secret_key,
 )
 from chinilla.wallet.transaction_record import TransactionRecord
+from chinilla.wallet.util.compute_memos import compute_memos
 from chinilla.wallet.util.transaction_type import TransactionType
-from chinilla.wallet.util.wallet_types import WalletType, AmountWithPuzzlehash
+from chinilla.wallet.util.wallet_types import AmountWithPuzzlehash, WalletType
 from chinilla.wallet.wallet import Wallet
 from chinilla.wallet.wallet_coin_record import WalletCoinRecord
 from chinilla.wallet.wallet_info import WalletInfo
-from chinilla.wallet.util.compute_memos import compute_memos
-import traceback
-
 
 # This should probably not live in this file but it's for experimental right now
 
@@ -97,8 +99,6 @@ class CATWallet:
 
         self.wallet_info = new_wallet_info
 
-        self.lineage_store = await CATLineageStore.create(self.wallet_state_manager.db_wrapper, self.get_asset_id())
-
         try:
             chinilla_tx, spend_bundle = await ALL_LIMITATIONS_PROGRAMS[
                 cat_tail_info["identifier"]
@@ -207,6 +207,23 @@ class CATWallet:
         await self.wallet_state_manager.add_new_wallet(self, self.id(), in_transaction=in_transaction)
         return self
 
+    @classmethod
+    async def create_from_puzzle_info(
+        cls,
+        wallet_state_manager: Any,
+        wallet: Wallet,
+        puzzle_driver: PuzzleInfo,
+        name=None,
+        in_transaction=False,
+    ) -> CATWallet:
+        return await cls.create_wallet_for_cat(
+            wallet_state_manager,
+            wallet,
+            puzzle_driver["tail"].hex(),
+            name,
+            in_transaction,
+        )
+
     @staticmethod
     async def create(
         wallet_state_manager: Any,
@@ -427,51 +444,35 @@ class CATWallet:
 
         return result
 
-    async def select_coins(self, amount: uint64) -> Set[Coin]:
+    async def select_coins(
+        self, amount: uint64, exclude: Optional[List[Coin]] = None, min_coin_amount: Optional[uint128] = None
+    ) -> Set[Coin]:
         """
         Returns a set of coins that can be used for generating a new transaction.
         Note: Must be called under wallet state manager lock
         """
-
-        spendable_am = await self.get_confirmed_balance()
-
-        if amount > spendable_am:
-            error_msg = f"Can't select amount higher than our spendable balance {amount}, spendable {spendable_am}"
-            self.log.warning(error_msg)
-            raise ValueError(error_msg)
-
-        self.log.info(f"About to select coins for amount {amount}")
-        spendable: List[WalletCoinRecord] = await self.get_cat_spendable_coins()
-
-        sum = 0
-        used_coins: Set = set()
-
-        # Use older coins first
-        spendable.sort(key=lambda r: r.confirmed_block_height)
+        spendable_amount: uint128 = await self.get_spendable_balance()
+        spendable_coins: List[WalletCoinRecord] = await self.get_cat_spendable_coins()
 
         # Try to use coins from the store, if there isn't enough of "unused"
         # coins use change coins that are not confirmed yet
         unconfirmed_removals: Dict[bytes32, Coin] = await self.wallet_state_manager.unconfirmed_removals_for_wallet(
             self.id()
         )
-        for coinrecord in spendable:
-            if sum >= amount and len(used_coins) > 0:
-                break
-            if coinrecord.coin.name() in unconfirmed_removals:
-                continue
-            sum += coinrecord.coin.amount
-            used_coins.add(coinrecord.coin)
-            self.log.info(f"Selected coin: {coinrecord.coin.name()} at height {coinrecord.confirmed_block_height}!")
 
-        # This happens when we couldn't use one of the coins because it's already used
-        # but unconfirmed, and we are waiting for the change. (unconfirmed_additions)
-        if sum < amount:
-            raise ValueError(
-                "Can't make this transaction at the moment. Waiting for the change from the previous transaction."
-            )
-
-        self.log.info(f"Successfully selected coins: {used_coins}")
-        return used_coins
+        coins = await select_coins(
+            spendable_amount,
+            self.wallet_state_manager.constants.MAX_COIN_AMOUNT,
+            spendable_coins,
+            unconfirmed_removals,
+            self.log,
+            uint128(amount),
+            exclude,
+            min_coin_amount,
+        )
+        assert coins is not None and len(coins) > 0
+        assert sum(c.amount for c in coins) >= amount
+        return coins
 
     async def sign(self, spend_bundle: SpendBundle) -> SpendBundle:
         sigs: List[G2Element] = []
@@ -734,7 +735,6 @@ class CATWallet:
             max_send = await self.get_max_send_amount()
             if payment_sum > max_send:
                 raise ValueError(f"Can't send more than {max_send} in a single transaction")
-
         unsigned_spend_bundle, chinilla_tx = await self.generate_unsigned_spendbundle(
             payments,
             fee,
@@ -743,7 +743,6 @@ class CATWallet:
             puzzle_announcements_to_consume=puzzle_announcements_to_consume,
         )
         spend_bundle = await self.sign(unsigned_spend_bundle)
-
         # TODO add support for array in stored records
         tx_list = [
             TransactionRecord(
@@ -810,3 +809,19 @@ class CATWallet:
         wallet_info = WalletInfo(current_info.id, current_info.name, current_info.type, data_str)
         self.wallet_info = wallet_info
         await self.wallet_state_manager.user_store.update_wallet(wallet_info, in_transaction)
+
+    def match_puzzle_info(self, puzzle_driver: PuzzleInfo) -> bool:
+        return (
+            AssetType(puzzle_driver.type()) == AssetType.CAT
+            and puzzle_driver["tail"] == bytes.fromhex(self.get_asset_id())
+            and puzzle_driver.also() is None
+        )
+
+    def get_puzzle_info(self, asset_id: bytes32) -> PuzzleInfo:
+        return PuzzleInfo({"type": AssetType.CAT.value, "tail": "0x" + self.get_asset_id()})
+
+    async def get_coins_to_offer(self, asset_id: Optional[bytes32], amount: uint64) -> Set[Coin]:
+        balance = await self.get_confirmed_balance()
+        if balance < amount:
+            raise Exception(f"insufficient funds in wallet {self.id()}")
+        return await self.select_coins(amount)
