@@ -8,12 +8,14 @@ from chinilla.wallet.lineage_proof import LineageProof
 from chinilla.types.blockchain_format.program import Program
 from chinilla.types.blockchain_format.coin import Coin
 
+DID_HRP = "did:chinilla:"
+
 
 @streamable
 @dataclass(frozen=True)
 class DIDInfo(Streamable):
     origin_coin: Optional[Coin]  # Coin ID of this coin is our DID
-    backup_ids: List[bytes]
+    backup_ids: List[bytes32]
     num_of_backup_ids_needed: uint64
     parent_info: List[Tuple[bytes32, Optional[LineageProof]]]  # {coin.name(): LineageProof}
     current_inner: Optional[Program]  # represents a Program as bytes
@@ -21,3 +23,4 @@ class DIDInfo(Streamable):
     temp_puzhash: Optional[bytes32]
     temp_pubkey: Optional[bytes]
     sent_recovery_transaction: bool
+    metadata: str  # JSON of the user defined metadata
