@@ -1,5 +1,3 @@
-import time
-
 import pytest
 
 from chinilla.protocols import full_node_protocol
@@ -7,11 +5,12 @@ from chinilla.types.peer_info import PeerInfo
 from chinilla.util.ints import uint16
 from tests.connection_utils import connect_and_get_peer
 from tests.time_out_assert import time_out_assert
+from tests.util.misc import assert_runtime
 
 
 class TestNodeLoad:
     @pytest.mark.asyncio
-    async def test_blocks_load(self, bt, two_nodes, self_hostname):
+    async def test_blocks_load(self, request: pytest.FixtureRequest, bt, two_nodes, self_hostname):
         num_blocks = 50
         full_node_1, full_node_2, server_1, server_2 = two_nodes
         blocks = bt.get_consecutive_blocks(num_blocks)
@@ -25,9 +24,9 @@ class TestNodeLoad:
 
         await time_out_assert(10, num_connections, 1)
 
-        start_unf = time.time()
-        for i in range(1, num_blocks):
-            await full_node_1.full_node.respond_block(full_node_protocol.RespondBlock(blocks[i]))
-            await full_node_2.full_node.respond_block(full_node_protocol.RespondBlock(blocks[i]))
-        print(f"Time taken to process {num_blocks} is {time.time() - start_unf}")
-        assert time.time() - start_unf < 100
+        with assert_runtime(seconds=100, label=request.node.name) as runtime_results_future:
+            for i in range(1, num_blocks):
+                await full_node_1.full_node.respond_block(full_node_protocol.RespondBlock(blocks[i]))
+                await full_node_2.full_node.respond_block(full_node_protocol.RespondBlock(blocks[i]))
+        runtime_results = runtime_results_future.result(timeout=0)
+        print(f"Time taken to process {num_blocks} is {runtime_results.duration}")
